@@ -12,7 +12,7 @@ GetOptions(
     'force'  => \$force,
 );
 
-die "Usage: analyse_cpan.pl --cpan path/to/minicpan --lint path/to/cpants_lint.pl" unless $cpan && $lint;
+die "Usage: run.pl --cpan path/to/minicpan --lint path/to/cpants_lint.pl" unless $cpan && $lint;
 die "Cannot find cpants_lint.pl (in $lint)" unless -e $lint;
 
 my $perl=$^X;
@@ -21,12 +21,12 @@ my $bin=catdir($home,'bin');
 $force='--force' if $force;
 
 system("$perl $bin/analyse_cpan.pl --cpan $cpan --lint $lint $force");
-system("$perl $bin/process_yaml.pl --cpan $cpan");
 system("$perl $bin/run_complex_db_stuff.pl --cpan $cpan");
 system("$perl $bin/update_authors.pl --cpan $cpan");
 system("$perl $bin/make_graphs.pl");
+
 #system($perl,"-I$lib", 
 #$path."/make_distgraph.pl",$site."root/static/graphs");
-#system($perl,"-I$lib", $path."/dump_sqlite.pl",$site."root/static/sqlite");
 
+system("$perl $bin/dump_sqlite.pl");
 
